# -*- coding: utf-8 -*-
# TencentBlueKing is pleased to support the open source community by making 蓝鲸智云-蓝鲸 PaaS 平台(BlueKing-PaaS) available.
# Copyright (C) 2017-2021 THL A29 Limited, a Tencent company. All rights reserved.
# Licensed under the MIT License (the "License"); you may not use this file except in compliance with the License.
# You may obtain a copy of the License at http://opensource.org/licenses/MIT
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
# an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.

"""
this command will sync the apis of current project to bk-apigateway

the related files for syncing:
- config/settings.py (the settings values for the gateway)
- definition.yaml (generated by command `generate_definition_yaml`, the template, will be rendered with the settings)
- resources.yaml (generated by command `generate_resource_yaml`, the api definitions of the gateway)

reference: https://github.com/TencentBlueKing/bkpaas-python-sdk/blob/master/sdks/apigw-manager/docs/sync-apigateway-with-django.md

the steps of syncing
1. generate definition.yaml:  `python manage.py generate_definition_yaml`
2. generate resources.yaml: `python manage.py generate_resources_yaml`
3. do the sync
"""

from pathlib import Path

from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    def handle(self, *args, **kwargs):
        gateway_name = settings.BK_APIGW_NAME

        file_dir = Path(settings.BASE_DIR)
        definition_file_path = file_dir / "definition.yaml"
        resources_file_path = file_dir / "resources.yaml"

        self.stdout.write(f"call sync_apigw_config with definition: {definition_file_path}")
        call_command(
            "sync_apigw_config",
            f"--gateway-name={gateway_name}",
            f"--file={definition_file_path}",
        )

        self.stdout.write(f"call sync_apigw_stage with definition: {definition_file_path}")
        call_command(
            "sync_apigw_stage",
            f"--gateway-name={gateway_name}",
            f"--file={definition_file_path}",
        )
        self.stdout.write(f"call sync_apigw_resources with resources: {resources_file_path}")
        call_command(
            "sync_apigw_resources",
            f"--gateway-name={gateway_name}",
            "--delete",
            f"--file={resources_file_path}",
            f"--doc_language={settings.BK_APIGW_RELEASE_DOC_LANGUAGE}",
        )

        # if BK_APIGW_RESOURCE_DOCS_BASE_DIR is not empty, then sync the resource docs
        if hasattr(settings, "BK_APIGW_RESOURCE_DOCS_BASE_DIR") and settings.BK_APIGW_RESOURCE_DOCS_BASE_DIR:
            self.stdout.write(f"call sync_resource_docs_by_archive with definition: {definition_file_path}")
            call_command(
                "sync_resource_docs_by_archive",
                f"--gateway-name={gateway_name}",
                f"--file={definition_file_path}",
                "--safe-mode",
            )

        self.stdout.write(f"call grant_apigw_permissions with definition: {definition_file_path}")
        call_command(
            "grant_apigw_permissions",
            f"--gateway-name={gateway_name}",
            f"--file={definition_file_path}",
        )

        self.stdout.write(
            f"call create_version_and_release_apigw with definition: {definition_file_path}, stage: {settings.BK_APIGW_STAGE_NAME}"
        )
        call_command(
            "create_version_and_release_apigw",
            f"--gateway-name={gateway_name}",
            f"--file={definition_file_path}",
            f"--stage={settings.BK_APIGW_STAGE_NAME}",
        )

        self.stdout.write(f"call fetch_apigw_public_key {gateway_name}")
        call_command("fetch_apigw_public_key", f"--gateway-name={gateway_name}")
